#ifndef B43_DEBUGFS_H_
#define B43_DEBUGFS_H_

struct b43_wldev;
struct b43_txstatus;

enum b43_dyndbg {		/* Dynamic debugging features */
	B43_DBG_XMITPOWER,
	B43_DBG_DMAOVERFLOW,
	B43_DBG_DMAVERBOSE,
	B43_DBG_PWORK_FAST,
	B43_DBG_PWORK_STOP,
	__B43_NR_DYNDBG,
};

#ifdef CONFIG_B43_DEBUG

struct dentry;

#define B43_NR_LOGGED_TXSTATUS	100

struct b43_txstatus_log {
	struct b43_txstatus *log;
	int end;
	int printing;
	char printbuf[(B43_NR_LOGGED_TXSTATUS * 70) + 200];
	size_t buf_avail;
	spinlock_t lock;
};

struct b43_dfsentry {
	struct dentry *subdir;
	struct dentry *dentry_tsf;
	struct dentry *dentry_txstat;
	struct dentry *dentry_txpower_g;
	struct dentry *dentry_restart;
	struct dentry *dentry_loctls;

	struct b43_wldev *dev;

	struct b43_txstatus_log txstatlog;

	/* Enabled/Disabled list for the dynamic debugging features. */
	u32 dyn_debug[__B43_NR_DYNDBG];
	/* Dentries for the dynamic debugging entries. */
	struct dentry *dyn_debug_dentries[__B43_NR_DYNDBG];
};

struct b43_debugfs {
	struct dentry *root;
	struct dentry *dentry_driverinfo;
};

int b43_debug(struct b43_wldev *dev, enum b43_dyndbg feature);

void b43_debugfs_init(void);
void b43_debugfs_exit(void);
void b43_debugfs_add_device(struct b43_wldev *dev);
void b43_debugfs_remove_device(struct b43_wldev *dev);
void b43_debugfs_log_txstat(struct b43_wldev *dev,
			    const struct b43_txstatus *status);

#else /* CONFIG_B43_DEBUG */

static inline int b43_debug(struct b43_wldev *dev, enum b43_dyndbg feature)
{
	return 0;
}

static inline void b43_debugfs_init(void)
{
}
static inline void b43_debugfs_exit(void)
{
}
static inline void b43_debugfs_add_device(struct b43_wldev *dev)
{
}
static inline void b43_debugfs_remove_device(struct b43_wldev *dev)
{
}
static inline
    void b43_debugfs_log_txstat(struct b43_wldev *dev,
				const struct b43_txstatus *status)
{
}

#endif /* CONFIG_B43_DEBUG */

/* Ugly helper macros to make incomplete code more verbose on runtime */
#ifdef TODO
# undef TODO
#endif
#define TODO()  \
	do {									\
		b43info(NULL, "TODO: Incomplete code in %s() at %s:%d\n",	\
			__FUNCTION__, __FILE__, __LINE__);			\
	} while (0)

#ifdef FIXME
# undef FIXME
#endif
#define FIXME()  \
	do {									\
		b43info(NULL, "FIXME: Possibly broken code in %s() at %s:%d\n",	\
			__FUNCTION__, __FILE__, __LINE__);			\
	} while (0)

#endif /* B43_DEBUGFS_H_ */
