/* Header File for In-System Design, Inc. USS725
 *
 * First release
 *
 * Current development and maintenance by:
 *   (c) 2002 Planel Nicolas (nplanel@mandrakesoft.com)
 *
 * See uss725.c for more information.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _USB_USS725_REGS_H
#define _USB_USS725_REGS_H

/*********************
 ** sequencing info **
 ********************/

/* define general register addresses */
#define REG_DATA 0x00
#define REG_STAT 0x01
#define REG_CTRL 0x02
#define REG_EPPA 0x03
#define REG_EPPD 0x04
#define REG_ECPA 0x05
#define REG_ECRR 0x06
#define REG_CCTR 0x07
#define REG_SETU 0x08
#define REG_TIMO 0x09
#define REG_ATAA 0x0A
#define NUMBER_OF_REGS 0x0B

/* define register addresses for read */
#define REG_STAT_R 0x00
#define REG_CTRL_R 0x01
#define REG_ECRR_R 0x02
#define REG_CCTR_R 0x03
#define REG_DATA_R 0x04
#define REG_EPPA_R 0x05
#define REG_EPPD_R 0x05
#define REG_SETU_R 0x06
#define REG_TIMO_R 0x07
#define REG_ATAA_R 0x08
#define NUMBER_OF_REGS_R 0x09

//
// REG_STAT register definition
//
#define REG_STAT_PPORTBUSY_OR_IOCS16    (0x80)
#define REG_STAT_PPORTACK_OR_IORDY      (0x40)
#define REG_STAT_PPORTERR_OR_PDIAG      (0x20)
#define REG_STAT_PPORTSEL_OR_INTRQ      (0x10)
#define REG_STAT_PPORTFAULT_OR_DASP     (0x08)
#define REG_STAT_1284_DISABLED          (0x04)
#define REG_STAT_PPORTPLH_OR_DMARQ      (0x02)
#define REG_STAT_EPP_TIMEOUT            (0x01)

//
// REG_SETU register definition
//
#define REG_SETU_HOLD_BULK              (0x80)
#define REG_SETU_TERMINATION_WAIT       (0x40)
#define REG_SETU_CLEAR_BULK_OUT         (0x20)
#define REG_SETU_FORCE_NEGOTIATION      (0x10)
#define REG_SETU_FORCE_COMPATIBILITY    (0x08)
#define REG_SETU_AUTO_HTR               (0x04)
#define REG_SETU_FILTER_SFT_OVERRIDE    (0x02)    
#define REG_SETU_FILTER_ENABLE          (0x01)

#define REG_SETU_STD                    (REG_SETU_HOLD_BULK|REG_SETU_FILTER_SFT_OVERRIDE)

//
// REG_CTRL register definition
//
#define REG_CTRL_PPORTHLH_RESET         (0x80)
#define REG_CTRL_EPP_MASK               (0x40)
#define REG_CTRL_DIRECTION_IN           (0x20) // else, is OUT
#define REG_CTRL_INT_ENABLE             (0x10)
#define REG_CTRL_PPORTSELECTIN_CSEL     (0x08)
#define REG_CTRL_PPORTNINIT_DMACK       (0x04)
#define REG_CTRL_PPORTAUTOFD_DIOR       (0x02)
#define REG_CTRL_PPORTSTROBE_DIOW       (0x01)

//
// REG_ECCR register definition
//
#define REG_ECCR_MODE_STANDARD          (0x00)
#define REG_ECCR_MODE_BIDIRECTIONAL     (0x20)
#define REG_ECCR_MODE_COMPATIBILITY     (0x40)
#define REG_ECCR_MODE_ECP               (0x60)
#define REG_ECCR_MODE_EPP               (0x80)
#define REG_ECCR_MODE_ATA               (0xE0)
#define REG_ECCR_NACK_INTR              (0x10)
#define REG_ECCR_NFAULT_INTR            (0x08)
#define REG_ECCR_BULKIN_INTR            (0x04)
#define REG_ECCR_BULKIN_EMPTY           (0x02)
#define REG_ECCR_BULKOUT_EMPTY          (0x01)

//
// REG_CCTR register definition
//
#define REG_CCTR_MANUAL_REVERSE         (0x80)
#define REG_CCTR_SLOW_RCCM              (0x40)
#define REG_CCTR_SLOW_INSTRUCTION       (0x20)
#define REG_CCTR_RETURN_ON_NO_DATA      (0x10)
#define REG_CCTR_NFAULT_INT_MASK        (0x08)
#define REG_CCTR_COMPRESS_ENABLE        (0x02)
#define REG_CCTR_AUTO_MODE              (0x01)

#define SEQ_RRMW 0x10
#define SEQ_RRCM 0x20
#define SEQ_RWRI 0xa0
#define SEQ_RREN 0x30
#define SEQ_WBIB 0x40
#define SEQ_WBOB 0x50
#define SEQ_DATI 0x60
#define SEQ_DATO 0x70
#define SEQ_EPPI 0x90
#define SEQ_NOOP 0x80
#define SEQ_SUBR 0xB0
#define SEQ_RETN 0xB0
#define SEQ_BLOCKING_BIT 0x80

/* commands/requests */
#define GET_DEVICE_ID		0 
#define GET_PORT_STATUS		1 
#define SOFT_RESET		2 
#define GET_1284_REGISTER	3 
#define SET_1284_REGISTER	4
#define GET_SEQ_STATUS	 	5
#define SEQ_LOAD		6
#define SEQ_ABORT		7
#define LOAD_EEPROM		8
#define CLEAR_BUFFER	 	9

// SEQUENCER_STATUS.statusBits
#define SEQSTATUSF_ACTIVE                   0x80
#define SEQSTATUSF_STOPPED                  0x40
#define SEQSTATUSF_EXECUTING_SUBROUTINE     0x20

struct seqstatus {
    unsigned char seqPos;
    unsigned char globalLoopCountHigh;
    unsigned char globalLoopCountLow;
    unsigned char statusFlags;
    unsigned char instructLoopCountMid;
    unsigned char instructLoopCountLow;
};

#endif
